package ABB_CC_Testcase "ABB Test Case Model Definition" 
  extends Modelica.Icons.Library;
  annotation (
    Coordsys(
      extent=[0, 0; 442, 394], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.45, 
      y=0.01, 
      width=0.44, 
      height=0.65, 
      library=1, 
      autolayout=1), 
    Documentation(info=" <HTML>
		<P>This Modelica package implements a test case that illustrates voltage stability 
			in power systems. It has been constructed within the framework of the 
			EU-sponsored Research program Control and Computation. Additional theoretical 
			background can be found in the textbooks on computer modelling power systems in 
			general (e.g. [1]) on voltage stability (e.g. [2,3]).


It can be used as a small-scale example for the development and testing of emergency control 
methods against voltage collapse. The model contains nonlinearity as well as hybrid behaviour and
can also be used to test algorithms. The test case will later be extended to medium and large
scale power systems and can thus also be used to study large-scale aspects of methods being evaluated.
</P>
		<P>Questions regarding the test case can be answered by Mats Larsson.</P>
		<P><B>REFERENCES</B></P>
		<P>[1] J. Machowski, J.W. Bialek, and J.R. Bumby, <I>Power System Dynamics and 
				Stability</I>, Number ISBN 0-471-97174. Wiley, 1993.</P>
		<P>[2] T. van Cutsem and C. Vournas, <I>Voltage Stability of Electric Power Systems</I>, 
			Number ISBN 0-7923-8139-4. Kluwer Academic Publishers, 1998.</P>
		<P>[3] M. Larsson, <A href=\"http://www.iea.lth.se/~ielmatsl/Publications/thesis.html\">Coordinated 
				Voltage Control in Power Systems</A>, PhD Thesis, Lund University, Sweden, 
			2000.</P>
		<P><STRONG>VERSION HISTORY</STRONG></P>
		<UL>
			<LI>
				v1.0 - first version of ABB Test Case Library, Feb 08, 2002.</LI></UL>
		<P><STRONG>AUTHOR</STRONG></P>
		<P>Mats Larsson, Corporate Research, ABB Schweiz AG,&nbsp;5405 Baden-Dttwil, 
			Switzerland.</P>
		<P>&nbsp;</P>
		<P>&nbsp;</P>
		<P>&nbsp;</P>

</html>
"));
  package Components 
    extends Modelica.Icons.Library2;
    
    model Slack "Slack Node" 
      extends Basic.OnePin;
      
      parameter Real V0=1;
      parameter Real theta0=0;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.21, 
          y=0.19, 
          width=0.6, 
          height=0.6), 
        Icon(
          Text(extent=[-40, -20; 80, 20], string="inf"), 
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Documentation(info="The slack node model acts as slack bus in initialization or load-flow 
calculations and as an infinite bus during dynamic simulation. 
In addition it defines the reference for the phasor calculation.



"), 
        Diagram);
    equation 
      1 + T.va = V0*cos(theta0);
      T.vb = V0*sin(theta0);
    end Slack;
    model Gen3 "3rd Order Detailed Generator" 
      extends Basic.OnePin;
      parameter Real H=3 "Inertia Constant";
      parameter Real D=0 "Damping Coefficient";
      
      parameter Real ra=0 "Armature Resistance";
      parameter Real xd=0.8948 "Direct Axis Synchronous Reactance";
      parameter Real xq=0.84 "Quadrature Axis Synchronous Reactance";
      parameter Real xdp=0.30 "Direct Axis Transient Reactance";
      parameter Real Td0p=7 "Open-circuit Direct Axis Transient Time Constant";
      
      parameter Real Ef0=1.72250079258051;
      parameter Real Efmax=2.2;
      
      parameter Real Pm0=0.352;
      parameter Real Pmmax=1;
      parameter Real wref=1;
      
      input Real Vref;
      
      Real Eqp(start=1);
      Real w(start=1) "Angular Speed";
      Real delta "Rotor Angle";
      
      Real id;
      Real iq "dq Armature Current";
      Real vd;
      Real vq "dq Armature Voltage";
      Real Pe;
      
      Real Efd;
      Real dEfd;
      Real Pm;
      Real dPm;
      Real V;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.33, 
          y=0.01, 
          width=0.67, 
          height=0.98), 
        Icon(
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Documentation(info="The 3rd order detailed generator model corresponds to Model 3 in [1, pp 348],
and extends the DetGen class. It adds a single transient EMF source in
the quatradure axis and the field voltage input. This model neglects the effect
of damper windings and the damping produced by rotor eddy-currents.
The effect of this damping is usually included in the damping coefficient D.

Each generator has it's own dq coordinate system, and its stator equations
must therefore be related to the network (global) coordinate system using the
so called Kron's transformations [1, pp. 90].



---
[1] J. Machowski, J.W. Bialek, and J.R. Bumby, Power System Dynamics and
Stability, Number ISBN 0-471-97174. Wiley, 1993.
"));
    equation 
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      
      //   swing equations
      der(w) = 1/(2*H)*(Pm - Pe - D*(w - 1));
      der(delta) = 2*Modelica.Constants.PI*50*(w - 1);
      
      // Transient EMF equation
      Td0p*der(Eqp) = Efd - Eqp + id*(xd - xdp);
      
      // stator equations
      vd = -ra*id - xq*iq;
      vq = Eqp + xdp*id - ra*iq;
      
      // electrical power
      Pe = Eqp*iq + (xdp - xq)*id*iq;
      
      // coordinate change
      -T.ia = -sin(delta)*id + cos(delta)*iq;
      -T.ib = cos(delta)*id + sin(delta)*iq;
      1 + T.va = -sin(delta)*vd + cos(delta)*vq;
      T.vb = cos(delta)*vd + sin(delta)*vq;
      
      // generator voltage control
      2.5*der(dEfd) + dEfd = 50*(Vref - V);
      Efd = min(Efmax, dEfd + Ef0);
      // governor control
      10*der(dPm) + dPm = 1.5*(wref - w);
      Pm = min(Pmmax, dPm + Pm0);
    end Gen3;
    model Load "Dynamic exponential recovery load" 
      extends Basic.OnePin;
      parameter Real P0=.1 "Rated Active Power Load";
      parameter Real Q0=.02 "Rated Reactive Power Load";
      parameter Real as=0 "Steady-state active power voltage dependency";
      parameter Real at=2 "Transient active power voltage dependency";
      parameter Real bs=0 "Steady-state reactive power voltage dependency";
      parameter Real bt=2 "Transient reactive power voltage dependency";
      parameter Real Tp=60 "Active power recovery time constant";
      parameter Real Tq=60 "Reactive power recovery time constant";
      parameter Real ShedStep=0.05;
      discrete input Integer step(min=0, max=3) "Load Shedding Constant";
      Real xp(min=-1, max=1) "Internal Load State";
      Real xq(min=-1, max=1) "Internal Load State";
      Real V(min=1.5, max=1.5);
      Real Pl(min=0, max=1);
      Real Ql(min=-0.5, max=1);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Line(points=[-40, -40; -20, -40; -20, -60; -6, -50; 6, -46; 24, -44; 
                40, -44]), 
          Polygon(points=[40, -2; -20, 18; -20, -22; 40, -2], style(
              color=0, 
              gradient=3, 
              fillColor=0)), 
          Line(points=[-100, 0; -20, 0], style(color=0)), 
          Text(extent=[-60, 60; 60, 20], string="%P0 + j %Q0")), 
        Window(
          x=0.12, 
          y=0.04, 
          width=0.6, 
          height=0.6), 
        Documentation(info="Dynamic exponential recovery load according to Karlsson & Hill [1].

The load powers are given by:

  der(xp) = P0*(V^(as) - V^at) - xp/Tp;
  Pl = (xp/Tp + P0*V^at);
  der(xq) = Q0*(V^(bs) - V^bt) - xq/Tq;
  Ql = (xq/Tq + Q0*V^bt);

where xp is a continuous dynamic state that can be interpreted as a
measure of the energy deficit in the load and Ps(V) = P0*V^as 
is the steady-state and Pt(V)=P0*V^at the transient voltage dependency.
Pl is the actual active load power and Tp is the active power recovery 
time constant. 

For the reactive load power, a similar model is used with corresponding 
characteristics x_q, Qs(V)=Q0 V^bs, Qt(V) = Q0 V^bt and time constant Tq.

---
[1] D. Karlsson and D.J. Hill, \"Modelling and identification of nonlinear 
dynamic loads in power systems\", IEEE Transactions on Power Systems, 
vol. 9, no. 1, pp. 157-163, February 1994.

"));
    equation 
      Pl = (1 - step*ShedStep)*(xp/Tp + P0*V^at);
      Ql = (1 - step*ShedStep)*(xq/Tq + Q0*V^bt);
      der(xp) = P0*(V^as - V^at) - xp/Tp;
      der(xq) = Q0*(V^bs - V^bt) - xq/Tq;
      
      T.ia = (T.vb*Ql + Pl + Pl*T.va)/(1 + 2*T.va + T.va*T.va + T.vb*T.vb);
      T.ib = (Pl*T.vb - Ql - T.va*Ql)/(1 + 2*T.va + T.va*T.va + T.vb*T.vb);
      
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
    end Load;
    model GenQSS "Quasi-Steady State Model of a generator" 
      extends Basic.OnePin;
      // typical limits for terminal variables
      // T.ia(min=-1.05, max=1.05), T.ib(min=-1.05, max=1.05)
      // T.va(min=-0.5, max=0.5  ), T.vb(min=-0.5,  max=0.5)
      parameter Real H(
        min=0.1, 
        max=20) = 3 "Inertia Constant";
      parameter Real D(
        min=0, 
        max=20) = 0 "Damping Coefficient";
      parameter Real ra(
        min=0.0, 
        max=0.02) = 0 "Armature Resistance";
      parameter Real xd(
        min=0.6, 
        max=2.4) = 0.8948 "Direct Axis Synchronous Reactance";
      parameter Real xq(
        min=0.4, 
        max=2.4) = 0.84 "Quadrature Axis Synchronous Reactance";
      parameter Real xdp(
        min=0.15, 
        max=1) = 0.30 "Direct Axis Transient Reactance";
      parameter Real Td0p(
        min=1.5, 
        max=10) = 7 "Open-circuit Direct Axis Transient Time Constant";
      
      parameter Real Ef0(
        min=1.5, 
        max=9) = 1.72250079258051;
      parameter Real Efmax=2.2;
      
      parameter Real Pm0=0.352;
      parameter Real Pmmax=1;
      parameter Real wref=1;
      
      input Real Vref(min=0.8, max=1.2);
      
      Real Eqp(
        start=1, 
        min=0, 
        max=2);
      Real w(
        start=1, 
        min=0.8, 
        max=1.2) "Angular Speed";
      Real delta(min=-Modelica.Constants.PI/2, max=Modelica.Constants.PI/2);
      
      Real id(min=-1.2, max=1.2);
      Real iq(min=-1.2, max=1.2);
      Real vd(min=-1.2, max=1.2);
      Real vq(min=-1.2, max=1.2);
      Real Pe(min=0, max=1.05);
      
      Real Efd(min=0, max=7);
      Real Pm(min=0, max=1.05);
      Real V(min=-0.5, max=1.5);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Window(
          x=0.18, 
          y=0, 
          width=0.7, 
          height=1), 
        Documentation(info="This model neglects the electromechanical dynamics of the
generator. 
"));
    equation 
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      
      //   swing equations
      0 = 1/(2*H)*(Pm - Pe - D*(w - 1));
      0 = 2*Modelica.Constants.PI*(w - 1);
      
      // Transient EMF equation
      Eqp = (Efd + id*(xd - xdp));
      
      // stator equations
      vd = -ra*id - xq*iq;
      vq = Eqp + xdp*id - ra*iq;
      
      // electrical power
      Pe = Eqp*iq + (xdp - xq)*id*iq;
      
      // coordinate change
      -T.ia = -sin(delta)*id + cos(delta)*iq;
      -T.ib = cos(delta)*id + sin(delta)*iq;
      1 + T.va = -sin(delta)*vd + cos(delta)*vq;
      T.vb = cos(delta)*vd + sin(delta)*vq;
      
      // generator voltage control
      Efd = min(Efmax, 50*(Vref - V) + Ef0);
      // governor control
      Pm = min(Pmmax, 1.5*(wref - w) + Pm0);
    end GenQSS;
    model Transformer "Transformer with Voltage Controller " 
      extends ImpTransformer;
      
      discrete input Real Vref(min=0.9, max=1.1);
      
      parameter Real TapDelay=30;
      parameter Real MechDelay=1;
      parameter Real TapStep=0.02;
      parameter Real TapDB=0.01*3;
      parameter Real MaxTap=10;
      parameter Real MinTap=-10;
      
      Boolean idle;
      Boolean wait(start=true);
      Boolean action;
      Boolean toohigh;
      Boolean toolow;
      discrete Real timer;
      discrete Integer tappos;
      
      Real V=sqrt((1 + T2.va)*(1 + T2.va) + T2.vb*T2.vb);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.31, 
          y=0, 
          width=0.61, 
          height=0.94), 
        Documentation(info="The tap changer controller is modelled as a state automata.
"));
    equation 
      // tap changer control - state automata !
      toohigh = (V - Vref) > TapDB/2;
      toolow = (V - Vref) < -TapDB/2;
      
      idle = (pre(idle) or pre(wait)) and not (toohigh or toolow) or (pre(
        action) and ((time - timer) > TapDelay + MechDelay));
      wait = (pre(idle) and (toohigh or toolow)) or pre(wait) and ((toolow or 
        toohigh) and ((time - pre(timer)) < TapDelay));
      action = (pre(wait) and (time - timer > TapDelay)) or pre(action) and ((
        time - timer) < TapDelay + MechDelay);
      
      when wait then
        timer = time;
      end when;
      
      when pre(action) and not action then
        if toolow and (pre(tappos) < MaxTap) then
          tappos = pre(tappos) + 1;
        elseif toohigh and (pre(tappos) > MinTap) then
          tappos = pre(tappos) - 1;
        else
          tappos = pre(tappos);
        end if;
      end when;
      Tr.n = 1 + TapStep*pre(tappos);
    end Transformer;
    class Varimp 
      extends Basic.TwoPin;
      
      parameter Real R=0.0 "Resistance";
      input Real X "Reactance";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Polygon(points=[4, 64; -50, -20; 20, 2; -24, -66; 50, 26; -30, -10; 4
                , 64], style(
              color=6, 
              fillColor=6, 
              fillPattern=1)), 
          Polygon(points=[24, 76; -30, -8; 38, 12; -4, -54; 70, 38; -10, 2; 24
                , 76], style(
              color=6, 
              gradient=2, 
              fillColor=1, 
              fillPattern=1)), 
          Text(extent=[34, 58; -46, 98], string="%name")), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    equation 
      [T1.va - T2.va; T1.vb - T2.vb] = [R, -X; X, R]*[T1.ia; T1.ib
        ];
      [T1.ia; T1.ib] + [T2.ia; T2.ib] = [0; 0];
    end Varimp;
    class Capacitor 
      extends Basic.OnePin;
      
      parameter Real G=0;
      parameter Real B=0.1;
      discrete input Integer step(min=0, max=3);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Line(points=[-20, 40; -20, -40], style(color=0)), 
          Line(points=[0, 40; 0, -40], style(color=0)), 
          Text(extent=[32, 46; -48, 86], string="%name"), 
          Line(points=[-100, 0; -20, 0], style(color=0))), 
        Window(
          x=0.19, 
          y=0.05, 
          width=0.6, 
          height=0.6));
    equation 
      [T.ia; T.ib] = step*[G, -B; B, G]*[1 + T.va; T.vb];
    end Capacitor;
    model IdealTransformer 
      extends Basic.TwoPin;
      Real n(start=1);
      Modelica.Blocks.Interfaces.InPort inPort(final n=1) annotation (extent=[-
            10, -90; 10, -70], rotation=90);
      annotation (
        Diagram, 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.01, 
          y=0.1, 
          width=0.6, 
          height=0.6), 
        Icon(
          Ellipse(extent=[-70, 40; 12, -40], style(color=0)), 
          Ellipse(extent=[-12, 40; 70, -40], style(color=0)), 
          Line(points=[70, 0; 100, 0], style(color=0)), 
          Line(points=[-100, 0; -70, 0], style(color=0)), 
          Text(extent=[-60, 80; 60, 40], string="%name"), 
          Text(extent=[-10, -78; -50, -38], string="1"), 
          Text(extent=[20, -76; -20, -36], string=":"), 
          Text(extent=[50, -76; 10, -36], string="n")));
    equation 
      
      (1 + T1.va)*n = 1 + T2.va;
      T1.vb*n = T2.vb;
      T1.ia = -T2.ia*n;
      T1.ib = -T2.ib*n;
      n = inPort.signal[1];
    end IdealTransformer;
    partial model ImpTransformer 
      extends Basic.TwoPin;
      parameter Real R=0.0 "Leakage Resistance";
      parameter Real X=0.1 "Leakage Reactance";
      annotation (
        Icon(
          Ellipse(extent=[-70, 40; 12, -40], style(color=0)), 
          Ellipse(extent=[-12, 40; 70, -40], style(color=0)), 
          Line(points=[70, 0; 100, 0], style(color=0)), 
          Line(points=[-100, 0; -70, 0], style(color=0)), 
          Text(extent=[-60, -40; 60, -80], string="%name")), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
      IdealTransformer Tr annotation (extent=[20, -20; 60, 20]);
      Impedance Imp(R=R, X=X) annotation (extent=[-60, -20; -20, 20]);
    equation 
      connect(Imp.T1, T1) annotation (points=[-60, 0; -100, 0]);
      connect(Imp.T2, Tr.T1) annotation (points=[-20, 0; 20, 0]);
      connect(Tr.T2, T2) annotation (points=[60, 0; 100, 0]);
    end ImpTransformer;
    annotation (
      Coordsys(
        extent=[0, 0; 442, 394], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.44, 
        height=0.65, 
        library=1, 
        autolayout=1), 
      Icon);
    package Basic 
      extends Modelica.Icons.Library;
      connector Pin "Electrical connector for ObjectStab models" 
        Real va;
        Real vb;
        flow Real ia;
        flow Real ib;
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(Rectangle(extent=[-20, 20; 20, -20], style(fillColor=73))), 
          Window(
            x=0.4, 
            y=0.4, 
            width=0.6, 
            height=0.6));
      end Pin;
      partial model OnePin 
        "Shell model for ObjectStab models with one electrical connector " 
        Pin T;
      end OnePin;
      partial model OnePinCenter 
        "Shell model for models with one connector in the center of the model" 
        Pin T annotation (extent=[-20, -20; 20, 20]);
        annotation (Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), Window(
            x=0.4, 
            y=0.4, 
            width=0.6, 
            height=0.6));
      end OnePinCenter;
      partial model TwoPin 
        "Shell model for ObjectStab models with two electrical connectors " 
        Pin T1;
        Pin T2;
      equation 
        
      end TwoPin;
      annotation (Coordsys(
          extent=[0, 0; 606, 355], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6, 
          library=1, 
          autolayout=1));
    end Basic;
    model Bus "Busbar model" 
      extends Basic.OnePinCenter;
      Real V;
      Real theta;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Icon(Rectangle(extent=[-12, 80; 10, -80], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), Text(extent=[80, -122; -80, -82], string="%name"
            )));
    equation 
      T.ia = 0;
      T.ib = 0;
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      theta = Modelica.Math.atan2(T.vb, (1 + T.va));
    end Bus;
    model Impedance "Impedance model" 
      extends Basic.TwoPin;
      
      parameter Real R=0.0 "Resistance";
      parameter Real X=0.1 "Reactance";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Icon(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Text(extent=[40, 20; -40, 60], string="%name")), 
        Documentation(info="The impedance model is governed by the equations:

           R+jX
           ----
V1, I1  ---    --- V2, I2
    ->     ----        <-


V1 - V2 = (R+jX) * I1
I1 + I2 = 0:


For numerical reasons, R and X may not both be set to zero.
"), 
        Diagram(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Text(extent=[40, 20; -40, 60], string="R+jX"), 
          Line(points=[-92, -10; -66, -10], style(arrow=1)), 
          Text(extent=[-60, -40; -100, -20], string="V1, I1"), 
          Text(extent=[100, -40; 60, -20], string="I2,V2"), 
          Line(points=[94, -12; 66, -12], style(arrow=1))));
    equation 
      [T1.va - T2.va; T1.vb - T2.vb] = [R, -X; X, R]*[T1.ia; T1.ib
        ];
      [T1.ia; T1.ib] + [T2.ia; T2.ib] = [0; 0];
    end Impedance;
  end Components;
  model System "Test case power system" 
    input Integer CapStep;
    input Integer LoadStep;
    input Real TapVref;
    input Boolean faulted;
    
    parameter Real G1Vref=1.03;
    
    output Real V2;
    output Real V3;
    output Real V4;
    output Real G1Efd;
    
    parameter Real L3X_1=0.5;
    parameter Real L3X_2=1.5;
    
    Components.Transformer T1(X=0.0031, tappos(start=1)) annotation (extent=[-
          20, -52; 20, -12], rotation=270);
    Components.Capacitor C3 annotation (extent=[-60, -62; -20, -20], rotation=
          270);
    Components.Varimp L3 annotation (extent=[-60, 20; -20, 60], rotation=90);
    Components.Bus Bus1 annotation (extent=[-80, 60; -40, 100]);
    Components.Bus Bus2 annotation (extent=[20, 60; 60, 100]);
    Components.Bus Bus3 annotation (extent=[-60, -20; 40, 20], rotation=90);
    Components.Bus Bus4 annotation (extent=[-20, -82; 20, -42], rotation=90);
    Components.Slack Ginf(V0=1.03) annotation (extent=[-100, 60; -60, 100], 
        rotation=180);
    Components.Impedance L1(X=0.75) annotation (extent=[-30, 60; 10, 100]);
    Components.Impedance L2(X=0.7) annotation (extent=[0, 20; 40, 60], rotation
        =90);
    Components.Load Load(
      P0=1, 
      Q0=0.2, 
      as=0, 
      at=2, 
      bs=1, 
      bt=2) annotation (extent=[-20, -102; 20, -62], rotation=270);
    replaceable Components.GenQSS G1(
      xd=2.38, 
      xq=2.27, 
      xdp=0.91, 
      Td0p=9.6) annotation (extent=[40, 60; 80, 100]);
  equation 
    connect(Bus4.T, T1.T2) annotation (points=[0, -62; -3.55271e-015
          , -52]);
    connect(C3.T, Bus3.T) annotation (points=[-40, -20; -40, 0; -10, 0]);
    connect(T1.T1, Bus3.T) annotation (points=[3.55271e-015, -12; 0, 0; -10, 0]
      );
    connect(L3.T1, Bus3.T) annotation (points=[-40, 20; -40, 0; -10, 0]);
    connect(L2.T1, Bus3.T) annotation (points=[20, 20; 20, 0; -10, 0]);
    annotation (
      Coordsys(
        extent=[-100, -102; 80, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.55, 
        height=0.99), 
      Diagram(Line(points=[0, -64; 0, -62])), 
      Icon(Rectangle(extent=[-80, 80; 60, -80], style(fillColor=72)), Text(
            extent=[-80, 60; 60, 20], string="Fournode")), 
      Documentation(info="The test case has three control inputs:

CapStep 
  Nr of capacitor steps connected at bus 3
  can assume the values [0,1,2,3]. Each step corresponds
  to 0.1 p.u. of compensation.
 
LoadStep
  The nr of load shedding steps used at bus 4
  can assume the values [0,1,2,3]. Each step corresponds

TapVref 
  The voltage reference for transformer T1
  By manipulating the Voltage reference, the
  load at bus 4 can be indirectly controlled since it is
  not completely reovereing following voltage reductions (as,bs > 0)

The test case has one disturbance input:
  
faulted
   Boolean variable indicating whether or not line L3 is faulted.
   The fault is modelled by the change of the impedance of the line
   from 1 p.u. to 1.5 p.u. 
  

The test case has three outputs:

V2,V3,V4
   The voltage at buses 2,3 and 4 respectively.
   The aim of emergency control is to keep the voltage of
   V4 within the limits [0.9 p.u. and 1.1 p.u.] despite
   the fault input becoming true. The control of voltages
   V2 and V3 is of secondary importance.


Recommended initial input values are:
CapStep=2, LoadStep=0, TapVref=1, and faulted=false (0)
"));
    connect(Bus1.T, Ginf.T) annotation (points=[-60, 80; -60, 80]);
    connect(L1.T2, Bus2.T) annotation (points=[10, 80; 40, 80]);
    connect(Bus2.T, L2.T2) annotation (points=[40, 80; 40, 72; 20, 72; 20, 60])
      ;
    connect(L3.T2, Ginf.T) annotation (points=[-40, 60; -40, 68; -40, 70; -60, 
          70; -60, 80]);
    connect(Ginf.T, L1.T1) annotation (points=[-60, 80; -36, 80; -30, 80]);
    connect(Load.T, Bus4.T) annotation (points=[3.55271e-015, -62; 0, -62]);
    connect(Bus2.T, G1.T) annotation (points=[40, 80; 40, 80]);
    C3.step = CapStep;
    Load.step = LoadStep;
    T1.Vref = TapVref;
    G1.Vref = G1Vref;
    G1.Efd = G1Efd;
    
    V2 = Bus2.V;
    V3 = Bus3.V;
    V4 = Bus4.V;
    
    L3.X = if faulted then L3X_2 else L3X_1;
  end System;
  model ExampleControl 
    "Test case power system with example emergency control applied" 
    parameter Integer strategy=0;
    parameter Real FaultTime=100;
    extends System(
      redeclare Real TapVref, 
      redeclare Integer CapStep, 
      redeclare Integer LoadStep, 
      redeclare Boolean faulted);
    
    parameter Real TimeOfControl=110;
    annotation (
      Coordsys(
        extent=[-100, -102; 80, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.02, 
        y=0, 
        width=0.47, 
        height=0.99), 
      Icon(Text(extent=[-60, -20; 40, -60], string="Controlled")), 
      Documentation(info="This model illustrates three possibilities of
emergency control. The aim of the emergeny control is
to stabilize the voltage of Bus 4 at a value higher than 0.9 
p.u.

It is trivial to design emergency control strategies
that will arrest the voltage collapse for a simple
with this simple structure based on heuristics.

strategy=0:
No emergency control, the voltage collapses after 568.5.

strategy=1:
Attempt to stabilize by change of transformer voltage control
setpoint, 

strategy=2:
Stabilitation by capacitor switching

strategy=3:
Stabilization by capacitor swithcing and tap reference
change

strategy=4:
Stabilization by load shedding


"));
  equation 
    if strategy == 0 then
      // no emergency control
      TapVref = 1;
      CapStep = 2;
      LoadStep = 0;
    elseif strategy == 1 then
      //attempt to stabilize by tap reference change
      TapVref = if time > TimeOfControl then 0.92 else 1;
      CapStep = 2;
      LoadStep = 0;
    elseif strategy == 2 then
      //attempt to stabilize by cap switching
      TapVref = 1;
      CapStep = if time > TimeOfControl then 3 else 2;
      LoadStep = 0;
    elseif strategy == 3 then
      //succesful stabilization by cap switching and tap reference change
      TapVref = if time > TimeOfControl then 0.92 else 1;
      CapStep = if time > TimeOfControl then 3 else 2;
      LoadStep = 0;
    elseif strategy == 4 then
      // stabilization by load shedding
      TapVref = 1;
      CapStep = 2;
      LoadStep = if time > TimeOfControl then 3 else 0;
    end if;
    
    faulted = if time > FaultTime then true else false;
  end ExampleControl;
end ABB_CC_Testcase;

